/**************************************************************************
**       Title: Kernel Matrix Write Application 
**    $RCSfile: KernelApplication.hh,v $
**   $Revision: 1.1 $$Name:  $
**       $Date: 2005/07/19 13:05:19 $
**   Copyright: GPL $Author: haasdonk $
** Description:
**
**-------------------------------------------------------------------------
**
**  $Log: KernelApplication.hh,v $
**  Revision 1.1  2005/07/19 13:05:19  haasdonk
**  initial revision
**
**
**************************************************************************/


#ifndef KERNELAPPLICATION_HH
#define KERNELAPPLICATION_HH

#include <string>
// #include <map>
#include <iostream>
#include <fstream>
#include <vector>

#include<StDataCmdLine.hh>
#include<HelpExtractor.hh>
#include<PrettyOptionPrinter.hh>
#include<BasicSVMAdapter.hh>
#include<TList.hh>
#include<BasicSVMFactory.hh>
#include <DefaultMultiClassList.hh>
#include <DefaultTwoClassList.hh>


// #include <cerrno>       // errno
// #include <cstring>      // strerror()

// #include "DirectAccessor.hh"

namespace svt
{

  /*======================================================================*/
  /*!
   *  \class KernelApplication
   *  \ingroup commandline
   *  \brief The KernelApplication class specifies a complete command line 
   *         application
   *
   *  see plain_to_dist_float.cc for an example how to use it
   *
   *  \param FV               feature vector class
   *  \param KFLIST           lists of kernel functions
   *  \param LOAD_SAVE_POLICY  policy for loading and saving of data
   */
  /*======================================================================*/
  template< typename FV,         
            typename KFLIST, 
            typename LOAD_SAVE_POLICY>
  class KernelApplication : public LOAD_SAVE_POLICY
  {
  public:
    typedef typename LOAD_SAVE_POLICY::STDATA STDATA;
    
    KernelApplication()
            :_prettyPrintColumn(30)
          {}
    

    virtual ~KernelApplication()
          {}

    /*======================================================================*/
    /*! 
     *   main function for a Kernel-Application. Call this if you don't
     *   want to go into details 
     *
     *   \param argc  number of command line arguments
     *   \param argv  array of c-style strings containing command line
     *                arguments 
     *   \param os    output stream for help text, progress messages, etc
     *
     *   \return 0: success 1: an error occured
     */
    /*======================================================================*/
    virtual int main( int argc, const char** argv, 
                      std::ostream& os = std::cout);
    
    /*======================================================================*/
    /*! 
     *   return the name of the shell program. This is the string from
     *   argv[0] by default, but could be replaced with
     *   setProgramName() 
     *
     *   \return program name
     */
    /*======================================================================*/
    virtual const std::string& programName() const
          {
            return _programName;
          }
    
    /*======================================================================*/
    /*! 
     *   Overwrite program name that was extracted from argv[0]. The
     *   programName() ist used from the several writeHelp() methods
     *
     *   \param name program name
     */
    /*======================================================================*/
    virtual void setProgramName( const std::string& name)
          {
            _programName = name;
          }

    /*======================================================================*/
    /*! 
     *   check for wrong ( = unused) parameters in cmdline
     *
     *   \param cmdline command line parameters
     *
     *   \exception svt::CmdLineError
     */
    /*======================================================================*/
    virtual void checkWrongParameters( StDataCmdLine& cmdline);
    

    /*======================================================================*/
    /*! 
     *   generate file(s) based on command line parameters
     *
     *   \param cmdline command line parameters
     *
     *   \exception 
     *
     *   \return 0: OK, otherwise some error occured and was reported
     *           to std::cerr 
     */
    /*======================================================================*/
    virtual int generateMatrixFile( StDataCmdLine& cmdline, 
                            std::ostream& os = std::cout);
    

    void setPrettyPrintColumn( int column)
          {
            _prettyPrintColumn = column;
          }
    

  private:
    std::string _programName;
    int _prettyPrintColumn;
       
  };
  
}

#include "KernelApplication.icc"


#endif
