/**************************************************************************
**       Title: convert libsvm compatible sparse data to netcdf-Files
**    $RCSfile: sparse_to_nc.cc,v $
**   $Revision: 1.1 $$Name:  $
**       $Date: 2004/08/26 08:36:59 $
**   Copyright: GPL $Author: ronneber $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: sparse_to_nc.cc,v $
**  Revision 1.1  2004/08/26 08:36:59  ronneber
**  initital import
**
**
**
**************************************************************************/
#include <iostream>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <cerrno>       // errno
#include <cstring>      // strerror()
#include <string>

#include <SparseFV.hh>
#include <BasicFV.hh>
#include <StDataNetCDF.hh>

int main( int argc, char** argv)
{
  if( argc != 3)
  {
    std::cerr << "usage: sparse_to_nc data.sparse filestub\n\n"
        "where filestub is the prefix for the netcdf-files. E.g., if\n"
        "filestub is 'obj_' the NetCDF-files will be named 'obj_00000.nc',\n"
        "'obj_00001.nc', 'obj_00002.nc', ...\n";
    return 1;
  }
  std::string inFileName = argv[1];
  std::string outFileStub = argv[2];
  

  /*-----------------------------------------------------------------------
   *  open input file
   *-----------------------------------------------------------------------*/
  std::ifstream inFile( inFileName.c_str());
  if( !inFile)
  {
    std::cerr << "error: Can't open '" << inFileName << "': " 
              << strerror(errno);
    exit( 1);
  }
  
  /*-----------------------------------------------------------------------
   *  read feature vectors one per line
   *-----------------------------------------------------------------------*/
  std::cout << "Loading feature vectors from '"<< inFileName << "'\n";
  std::vector<svt::SparseFV> sparseFVs;
  while( inFile.good())
  {
    //skip spaces and find eof
    char c;
    while (inFile.good() && inFile.get(c) && isspace(c)) 
    {}
    
    if (!inFile.good())
    {
      break;
    }
    else if (!isspace(c))
    {
      inFile.putback(c);
    }
    
    svt::SparseFV fv;
    inFile >> fv;
    sparseFVs.push_back(fv);
  }

  std::cout << "Found " << sparseFVs.size() << " feature vectors\n";

  /*-----------------------------------------------------------------------
   *  find out maximum index
   *-----------------------------------------------------------------------*/
  std::cout << "Searching for feature vectors maximum size\n";
  unsigned int maxIndex = 0;
  for( std::vector<svt::SparseFV>::const_iterator p = sparseFVs.begin();
       p != sparseFVs.end(); ++p)
  {
    svt::SparseFV::const_iterator q = p->end();
    --q;
    if( q->index() > maxIndex) maxIndex = q->index();
  }
  
  std::cout << "maximum index is: " << maxIndex << "\n";
  
  /*-----------------------------------------------------------------------
   *  Create output NetCDF files and write feature vectors
   *-----------------------------------------------------------------------*/
  svt::BasicFV fv;
  fv.resize( maxIndex);
  for( unsigned int i = 0; i < sparseFVs.size(); ++i)
  {
    std::ostringstream oss;
    oss << outFileStub << std::setfill('0') << std::setw(5) << i
        << ".nc";
    
    std::cout << "saving feature vector " << i << " to '" << oss.str() << "'\n";
    
    std::vector<svt::SparseFV>::const_iterator p = sparseFVs.begin() + i;
    double label = p->getLabel();
    for( unsigned int i = 1; i <= maxIndex; ++i)
    {
      fv[i-1] = (*p)[i];
    }

    svt::StDataNetCDF outFile( oss.str().c_str(), svt::StDataNetCDF::Replace);
    outFile.setValue( "code", label);
    outFile.setArray( "fv", fv.begin(), fv.size());
    
  }

  return 1;
}

  
