//
//  Test string to template function

#include <iostream>
#include <string>
#include <map>

// SVM parent class
class SVM
{
public:
  virtual void train() { std::cout << "SVM::train"; };
};


// Specialized SVM with given algorithms
template< typename MC, typename TC, typename KF>
class SpecialSVM : public SVM
{
public:
  void train()
        {
          std::cout << "SepcialSVM::train with "
                    << MC::name() << ", " 
                    << TC::name() << ", "
                    << KF::name() << std::endl;
        }
};

// Multi class algorithms
class MC1 { public: static std::string name(){ return "MC1";} };
class MC2 { public: static std::string name(){ return "MC2";} };
class MC3 { public: static std::string name(){ return "MC3";} };
class MC4 { public: static std::string name(){ return "MC4";} };

// Two class algorithms
class TC1 { public: static std::string name(){ return "TC1";} };
class TC2 { public: static std::string name(){ return "TC2";} };
class TC3 { public: static std::string name(){ return "TC3";} };
class TC4 { public: static std::string name(){ return "TC4";} };

// Kernel functions
class KF1 { public: static std::string name(){ return "KF1";} };
class KF2 { public: static std::string name(){ return "KF2";} };
class KF3 { public: static std::string name(){ return "KF3";} };
class KF4 { public: static std::string name(){ return "KF4";} };

// template< typename T1, typename T2, typename T3, typename T4>
// class myList
// {
// public:
//   myList( std::string key)
//           :_key(key)
//         {}
//   
//   template<typename Functor>
//   SVM* select1( const std::map<std::string, std::string>& param, Functor F)
//         {
//           if( T1::name() == key) return F<T1>(param);
//           if( T2::name() == key) return F<T2>(param);
//           if( T3::name() == key) return F<T3>(param);
//           if( T4::name() == key) return F<T4>(param);
//         }
//   
//   template<typename TT1, typename Functor>
//   SVM* select2( const std::map<std::string, std::string>& param, Functor F)
//         {
//           if( T1::name() == key) return F<TT1, T1>(param);
//           if( T2::name() == key) return F<TT1, T2>(param);
//           if( T3::name() == key) return F<TT1, T3>(param);
//           if( T4::name() == key) return F<TT1, T4>(param);
//         }
//   
//   template<typename TT1, typename TT2, typename Functor>
//   SVM* select3( const std::map<std::string, std::string>& param, Functor F)
//         {
//           if( T1::name() == key) return F<TT1, TT2, T1>(param);
//           if( T2::name() == key) return F<TT1, TT2, T2>(param);
//           if( T3::name() == key) return F<TT1, TT2, T3>(param);
//           if( T4::name() == key) return F<TT1, TT2, T4>(param);
//         }
//   
// private:
//   std::string _key;
// };
//           
// 
// template<typename MCL, typename TCL, typename KFL>
// class SVMFactory
// {
// public:
// 
//   SVMFactory( const MCL& mcl, const TCL& tcl, const KFL& kfl)
//           :_mcl(mcl), _tcl(tcl), _kfl(kfl)
//         {}
//   
// 
//   template<typename T1, typename T2, typename T3>
//   SVM* create_Special_SVM( const std::map<std::string, std::string>& param)
//         {
//           return new SpecialSVM<T1, T2, T3>;
//         }
// 
//   template<typename T1, typename T2>
//   SVM* select_Kernel(const std::map<std::string, std::string>& param)
//         {
//           return _kfl.select3<T1, T2, create_Special_SVM>(param);
//         }
//   
//   template<typename T1>
//   SVM* select_TwoClass(const std::map<std::string, std::string>& param)
//         {
//           return _tcl.select2<T1, select_Kernel>(param);
//         }
//   
// 
// 
//   SVM* select_MultiClass(const std::map<std::string, std::string>& param)
//         {
//           return _kfl.select1(param, select_TwoClass);
//         }
// 
// 
// private:
//   MCL _mcl;
//   TCL _tcl;
//   KFL _kfl;
// 
// };
// 
// 
// 
// 

template<typename T>
void myfunc()
{
  std::cout << T::name() << std::endl;
}


template<template<typename> class T>
void callFunctor(T func)
{
  func<MC1>();
}




int main( int argc, char**argv)
{
  // Create SVM with requested algorithms (hard coded)
  SVM* svm = new SpecialSVM< MC1, TC3, KF2>;

  // train SVM
  svm->train();


  // and now dynamic selection, parameters come e.g. from commandline
  std::map<std::string, std::string> param;
  param["multiclass"] = "MC1";
  param["twoclass"] = "TC3";
  param["kernel"] = "KF2";
  
//  // register my algorithms and kernel functions somewhere
//  myList<MC1, MC2, MC3, MC4> mc_List("multiclass");
//  myList<TC1, TC2, TC3, TC4> tc_List("twoclass");
//  myList<KF1, KF2, KF3, KF4> kf_List("kernel");
//
//  // now I want an SVM with the parameters given in param
//  SVMFactory< 
//      myList<MC1, MC2, MC3, MC4>,
//      myList<TC1, TC2, TC3, TC4>,
//      myList<KF1, KF2, KF3, KF4> > svmFac( mc_List, tc_List, kf_List);
//  
//  svm = svmFac.select_MultiClass(param);

  myfunc<MC1>();
  
  callFunctor( myfunc);
  
  

  return 0;
}
