#include <iostream>
#include <map>

#include "../libsvmtl/Kernel_LINEAR.hh"
#include "../libsvmtl/Kernel_POLY.hh"
#include "../libsvmtl/Kernel_RBF.hh"
#include "../libsvmtl/Kernel_SIGMOID.hh"
#include "../libsvmtl/Kernel_MATRIX.hh"
#include "../libsvmtl/TList.hh"
#include "../libsvmtl/TwoClassSVMc.hh"
#include "../libsvmtl/TwoClassSVMnu.hh"
#include "../libsvmtl/MultiClassSVMOneVsOne.hh"
#include "../libsvmtl/MultiClassSVMOneVsRest.hh"

  typedef svt::MakeTTList<
      svt::MultiClassSVMOneVsOne, 
      svt::MultiClassSVMOneVsRest> MultiClassList;

typedef svt::MakeTList<
    svt::Kernel_LINEAR, 
    svt::Kernel_RBF, 
    svt::Kernel_POLY, 
    svt::Kernel_SIGMOID,
    svt::Kernel_MATRIX<svt::Kernel_LINEAR>,
    svt::Kernel_MATRIX<svt::Kernel_RBF>,
    svt::Kernel_MATRIX<svt::Kernel_POLY>,
    svt::Kernel_MATRIX<svt::Kernel_SIGMOID> > KernelList;


typedef svt::MakeTTList<
    svt::TwoClassSVMc, 
    svt::TwoClassSVMnu> TwoClassList;
 

// base class
template< typename ITER>
struct HelpExtractor
{
};

// partial specialization for TList's
template<typename X, typename Y>
struct HelpExtractor<svt::TList<X,Y> >
{

  typename svt::TList<X,Y> ITER;
  
  static void getParamHelp( std::map<std::string, std::set<std::string> >& help)
        {
          svt::TList<X,Y>::val_t::getParamHelp( help);
          
          HelpExtractor<typename svt::TList<X,Y>::next_t>::getParamHelp( help);
        }
  static void getNamesDescriptions( std::map<std::string, std::string>& 
                                    nameDescriptionMap)
        {
          nameDescriptionMap[svt::TList<X,Y>::val_t::name()] = svt::TList<X,Y>::val_t::description();
          HelpExtractor<typename svt::TList<X,Y>::next_t>::getNamesDescriptions( 
              nameDescriptionMap);
          
        }
};


// partial specialization for TTList's
template<template<typename> class X, typename Y>
struct HelpExtractor<svt::TTList<X,Y> >
{

  typename svt::TTList<X,Y> ITER;
  
  static void getParamHelp( std::map<std::string, std::set<std::string> >& help)
        {
          svt::TTList<X,Y>::Traits<int>::val_t::getParamHelp( help);
          
          HelpExtractor<typename svt::TTList<X,Y>::next_t>::getParamHelp( help);
        }
  static void getNamesDescriptions( std::map<std::string, std::string>& 
                                    nameDescriptionMap)
        {
          nameDescriptionMap[svt::TTList<X,Y>::Traits<int>::val_t::name()] = svt::TTList<X,Y>::Traits<int>::val_t::description();
          HelpExtractor<typename svt::TTList<X,Y>::next_t>::getNamesDescriptions( nameDescriptionMap);
          
        }
};

// partial specialization for TList's end
template<typename X>
struct HelpExtractor<svt::TList<svt::TList_end,X> >
{
  static void getParamHelp( std::map<std::string, std::set<std::string> >& help)
        {
        }

  static void getNamesDescriptions( std::map<std::string, std::string>&
                                    nameDescriptionMap)
        {
        }
};

// partial specialization for TTList's end
template<typename X>
struct HelpExtractor<svt::TTList<svt::TTList_end,X> >
{
  static void getParamHelp( std::map<std::string, std::set<std::string> >& help)
        {
        }

  static void getNamesDescriptions( std::map<std::string, std::string>&
                                    nameDescriptionMap)
        {
        }
};



template< typename ITER>
void testHelpExtractor()
{
  std::map<std::string, std::set<std::string> > help;
  HelpExtractor< ITER>::getParamHelp( help);

  for( std::map<std::string, std::set<std::string> >::const_iterator p = 
           help.begin(); p != help.end(); ++p)
  {
    for( std::set<std::string>::const_iterator 
             q = p->second.begin(); q != p->second.end(); ++q)
    {
      std::cout << "--" << p->first << "\t" << *q << std::endl;
    }
  }


  std::map<std::string,std::string> nameDescriptionMap;
  HelpExtractor< ITER>::getNamesDescriptions( nameDescriptionMap);
  for( std::map<std::string,std::string>::const_iterator p =
           nameDescriptionMap.begin(); p != nameDescriptionMap.end(); ++p)
  {
    std::cout << p->first << "\t" << p->second << std::endl;
  }
}




int main( int argc, char** argv)
{
  testHelpExtractor<KernelList::the_TList>();
  testHelpExtractor<TwoClassList::the_TTList>();
  testHelpExtractor<MultiClassList::the_TTList>();
  
  return 0;
}
