//
//  Test string to template function

#include <iostream>
#include <string>
#include <map>
#include <exception>

// SVM parent class
class SVM
{
public:
  virtual void train() { std::cout << "SVM::train"; };
};


// Specialized SVM with given algorithms
template< typename MC, typename TC, typename KF>
class SpecialSVM : public SVM
{
public:

  void train()
        {
          std::cout << "SepcialSVM::train with "
                    << MC::name() << ", "
                    << TC::name() << ", "
                    << KF::name() << std::endl;
        }
};

template<typename T, typename Tnext> struct tlist {
  typedef T val_t;
  typedef Tnext next_t;
};

struct tlist_end {};


// Multi class algorithms
class MC1 { public: static std::string name(){ return "MC1";} };
class MC2 { public: static std::string name(){ return "MC2";} };
class MC3 { public: static std::string name(){ return "MC3";} };
class MC4 { public: static std::string name(){ return "MC4";} };
typedef tlist <MC1, tlist <MC2, tlist <MC3, tlist <MC4, tlist<tlist_end, tlist_end> > > > >
  multi_class_list_t;


// Two class algorithms
class TC1 { public: static std::string name(){ return "TC1";} };
class TC2 { public: static std::string name(){ return "TC2";} };
class TC3 { public: static std::string name(){ return "TC3";} };
class TC4 { public: static std::string name(){ return "TC4";} };

typedef tlist <TC1, tlist <TC2, tlist <TC3, tlist <TC4, tlist<tlist_end, tlist_end> > > > >
  two_class_list_t;


// Kernel functions
class KF1 { public: static std::string name(){ return "KF1";} };
class KF2 { public: static std::string name(){ return "KF2";} };
class KF3 { public: static std::string name(){ return "KF3";} };
class KF4 { public: static std::string name(){ return "KF4";} };

typedef tlist <KF1, tlist <KF2, tlist <KF3, tlist <KF4, tlist<tlist_end, tlist_end> > > > >
  kernel_function_list_t;


template<typename T1, typename T2, typename T3>
struct SVMCreator
{
  static SVM* create()
        {
          return new SpecialSVM<T1,T2,T3>;
        }
};

#define SVMFACTORY_FORBID_CLASS_COMBINATION( __CLASS1, __CLASS2, __CLASS3) template<typename X> struct SVMCreator<__CLASS1, __CLASS2, __CLASS3> { static SVM* create() { throw (std::string ("nonono")); } };

template<typename T1, typename T2, typename T3>
struct select_classes 
{
  static SVM* select (std::string m,
                      std::string t,
                      std::string k) 
        {
          if (m != T1::val_t::name()) 
          {
            return select_classes <typename T1::next_t, T2, T3>::select (m, t, k);
          }
          else if (t != T2::val_t::name()) 
          {
            return select_classes <T1, typename T2::next_t, T3>::select (m, t, k);
          }
          else if (k == T3::val_t::name()) 
          {
            return select_classes <T1, T2, typename T3::next_t>::select (m, t, k);
          }
          else
          {
            return SVMCreator<typename T1::val_t,
                typename T2::val_t, typename T3::val_t>::create();
            
          } 
        }
};

template<typename A, typename B, typename C> 
struct select_classes< tlist<tlist_end,A>, B, C> 
{
  static SVM* select (std::string m, std::string t, std::string k) 
        {
          throw (std::string ("damn"));
        }
};

template<typename A, typename B, typename C> 
struct select_classes<A, tlist<tlist_end,B>, C> 
{
  static SVM* select (std::string m, std::string t, std::string k) 
        {
          throw (std::string ("damn"));
        }
};

template<typename A, typename B, typename C> 
struct select_classes<A, B, tlist<tlist_end,C> > 
{
  static SVM* select (std::string m, std::string t, std::string k) 
        {
          throw (std::string ("damn"));
        }
};


//Don't allow combination of MC1 with TC2
SVMFACTORY_FORBID_CLASS_COMBINATION( MC1, TC2, X);


template< typename MCLIST, typename TCLIST, typename KFLIST>
SVM* createSpecialSVM (std::map<std::string, std::string>& param) {
  std::cout << "searching (" << param["multiclass"] << ", "
            << param["twoclass"] <<  ", "
            << param["kernel"] <<  ")" << std::endl;

  return select_classes<
      typename MCLIST::the_list,
      typename TCLIST::the_list,
      typename KFLIST::the_list>::select (param["multiclass"],
                             param["twoclass"],
                             param["kernel"]);
}


template<typename T1, typename T2=tlist_end, typename T3=tlist_end, typename T4=tlist_end>
struct make_list
{
  typedef tlist <T1, tlist <T2, tlist <T3, tlist <T4, tlist< tlist_end, tlist_end> > > > > the_list;
};





int main( int argc, char**argv)
{
  // Create SVM with requested algorithms (hard coded)
  try
  {
    SVM* svm = new SpecialSVM< MC1, TC3, KF2>;
    
    // train SVM
    svm->train();
    
    
    // and now dynamic selection, parameters come e.g. from commandline
    std::map<std::string, std::string> param;
    param["multiclass"] = "MC1";
    param["twoclass"] = "TC3";
    param["kernel"] = "KF4";

    // register my algorithms and kernel functions somewhere
    // ....
    // now I want an SVM with the parameters given in param
    svm = createSpecialSVM<
        make_list<MC1, MC2, MC3, MC4>,
        make_list<TC1, TC2, TC3, TC4>,
        make_list<KF1, KF2, KF3, KF4> >(param);
    
    // train SVM
    svm->train()

;
    // check if forbidden Combinations are not instantiated
    param["multiclass"] = "MC1";
    param["twoclass"] = "TC2";
    param["kernel"] = "KF4";

    // register my algorithms and kernel functions somewhere
    // ....
    // now I want an SVM with the parameters given in param
    svm = createSpecialSVM<
        make_list<MC1, MC2, MC3, MC4>,
        make_list<TC1, TC2, TC3, TC4>,
        make_list<KF1, KF2, KF3, KF4> >(param);
    
    // train SVM
    svm->train();
  }
  catch (const std::string& error)
  {
    std::cerr << "Error: " << error << std::endl;
  }
  
  



  return 0;
}
