/**************************************************************************
**       Title: experimental (hopyfully faster) version of svmtl
**    $RCSfile: svmtl256.cc,v $
**   $Revision: 1.2 $$Name:  $
**       $Date: 2005/01/14 16:17:38 $
**   Copyright: GPL $Author: fehr $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: svmtl256.cc,v $
**  Revision 1.2  2005/01/14 16:17:38  fehr
**  64-Bit Port and netCDF upgrade 3.5->3.6
**
**  Revision 1.1  2004/08/26 08:36:59  ronneber
**  initital import
**
**
**
**************************************************************************/

#include <SVMApplication.hh>
#include <BasicFV.hh>
#include <AlgorithmLists.hh>
#include <DefaultMultiClassList.hh>
#include <DefaultTwoClassList.hh>
#include <DefaultOneClassList.hh>
#include <DefaultKernelList.hh>
#include <LoadSaveASCII.hh>

// feature vector class with exactly 256 dimensions
class FeatureVector256
{
public:
  typedef std::vector<double>::iterator iterator;
  typedef std::vector<double>::const_iterator const_iterator;
  typedef std::vector<double>::reference reference;
  typedef std::vector<double>::const_reference const_reference;
  typedef std::vector<double>::size_type size_type;
		  
  FeatureVector256()
          :_uniqueID( MAX_BELIEVABLE_UNIQUE_ID + 1),
           _label(0),
           _square(-1)
        {}
  
  double getLabel() const
        {
          return _label;
        }
  
  void setUniqueID( unsigned int uid)
        {
          _uniqueID = uid;
        }
  
  unsigned int uniqueID() const
        {
          return _uniqueID;
        }
  
  double square() const
        {
          return _square;
        };
 
  reference operator[](
        int index)
          {
            return _data[index];
          };
 const_reference operator[](
        int index) const
          {
            return _data[index];
          }; 
 
  size_type size() const
          {
            return 256;
          };

 
  double dotProduct( const FeatureVector256& fv) const
          {
            double sum=0.;

            const double* p1=_data;
            const double* p2=fv._data;
            for ( unsigned int i = 0; i < _size; ++i)
            {
              sum+=(*p1) * (*p2);
              ++p1;
              ++p2;
            }

            return sum;
          };

  static const char* helpPipeFormat()
          {
            return "<label><ws><feature_0><ws><feature_1>...<ws><feature_n>\n"
                "where <ws> is any number of white spaces except for newline\n"
                "example:\n"
                "4 0.123 2.432 42.0 137.0815 24.35";
          }

 
    friend std::ostream& operator<<(std::ostream& os, const FeatureVector256& fv);
    friend std::istream& operator>>(std::istream& is, FeatureVector256& fv);

private:
  static const int _size = 256;
  double       _data[_size];
  unsigned int _uniqueID;
  double       _label;
  double       _square;
};

inline
std::ostream& operator<<(std::ostream& os, const FeatureVector256& fv)
{
  os << fv._label;
  for( unsigned int i = 0; i < fv._size; ++i)
  {
    os << " " << fv._data[i];
  }
  return os;
}


inline 
std::istream& operator>>(std::istream& is, FeatureVector256& fv)
{
  // read label
  is >> fv._label;
  fv._square = 0;
  for( unsigned int i = 0; i < fv._size; ++i)
  {
    is >> fv._data[i];
    fv._square += fv._data[i]*fv._data[i];
  }
  return is;
}

int main( int argc, const char** argv)
{
  svt::SVMApplication< 
      FeatureVector256,
      svt::AlgorithmLists< svt::DefaultMultiClassList,
                           svt::DefaultTwoClassList,
                           svt::DefaultOneClassList,
                           svt::DefaultKernelList>,
      svt::LoadSaveASCII> sa; 
		  
  return sa.main( argc, argv, std::cout);
}

